# -*- coding: utf-8 -*-

# This is the item picker panel in the main
# application's notebook.

"""
Copyright (C) 2006-2008 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import sys
try:
    import Image
except:
    print "\nPlease install the Python Imaging Library (PIL)\n"
    sys.exit()
import wx
import os
import re
import platform
import sre_constants
import utils
import time

[wxID_PICKERPANEL, wxID_PICKERPANELBROWSE, wxID_SELECTIONAREADIRPICKER,
 wxID_PICKERPANELDIVIDER1, wxID_PICKERPANELFILESON,
 wxID_PICKERPANELFILTERSEL, wxID_PICKERPANELFOLDERSON, wxID_PICKERPANELNOT_TYPE,
 wxID_PICKERPANELOK, wxID_PICKERPANELPATH, wxID_PICKERPANELPATH_HELP,
 wxID_SELECTIONAREAFPICKER, wxID_PICKERPANELSELECT, wxID_PICKERPANELSELECT_ALL,
 wxID_PICKERPANELSELECT_NONE, wxID_PICKERPANELSTATICTEXT1,
 wxID_PICKERPANELWALKIT, wxID_PICKERPANELFILTERBYRE, wxID_SELECTIONAREA,
wxID_PICKERPANELIGNORECASE, wxID_PICKERPANELUSELOCALE
] = [wx.NewId() for _init_ctrls in range(21)]

class DirControl(wx.GenericDirCtrl):
    def __init__(self, parent, ID, name, showHidden):
        self.parent = parent.GetParent()
        wx.GenericDirCtrl.__init__(self, parent, ID, name,
          defaultFilter=0, filter='', style=wx.DIRCTRL_DIR_ONLY)
        
        if showHidden:
            self.ShowHidden(True)

        isz = (16,16)
        il = wx.ImageList(isz[0], isz[1])
        # closed folder:
        il.Add(wx.Bitmap(utils.realPath(u'icons/folder16.png'),wx.BITMAP_TYPE_PNG))
        # open folder:
        il.Add(wx.Bitmap(utils.realPath(u'icons/folderOpen.png'),wx.BITMAP_TYPE_PNG))
        # root of filesystem (linux):
        il.Add(wx.Bitmap(utils.realPath(u'icons/root.ico'),wx.BITMAP_TYPE_ICO))
        # drive letter (windows):
        il.Add(wx.Bitmap(utils.realPath(u'icons/hdd.ico'),wx.BITMAP_TYPE_ICO))
        # cdrom drive:
        il.Add(wx.Bitmap(utils.realPath(u'icons/cdrom.ico'),wx.BITMAP_TYPE_ICO))
        # removable drive on win98:
        il.Add(wx.Bitmap(utils.realPath(u'icons/fdd.ico'),wx.BITMAP_TYPE_ICO))
        # removable drive (floppy, flash, etc):
        il.Add(wx.Bitmap(utils.realPath(u'icons/fdd.ico'),wx.BITMAP_TYPE_ICO))
        self.il = il
        self.GetTreeCtrl().SetImageList(il)
        self.bind()

    def dirChange(self,event):
        self.parent.path.SetValue(self.GetPath())
        self.parent.ShowPicker(event)
    
    def unbind(self):
        self.Bind(wx.EVT_TREE_SEL_CHANGED, utils.devNull)
    
    def bind(self):
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.dirChange)


# picker list and associated functions
class picker(wx.ListCtrl):
    def __init__(self, parent, id, name):
        self.pickerPanel = parent.GetParent()

        style = wx.LC_LIST
        #style = wx.LC_REPORT|wx.LC_NO_HEADER|wx.LC_VIRTUAL

        wx.ListCtrl.__init__(self, parent, id=id, name=name, style=style)
        self.thumbnails = {}
        self.CreateImageList(main.bottomWindow.imgPreview.GetValue())

        """
        self.SetItemCount(100000)
        self.InsertColumn(0, "First")
        self.SetColumnWidth(0, 470)

        self.attr1 = wx.ListItemAttr()
        self.attr1.SetBackgroundColour("yellow")

        self.attr2 = wx.ListItemAttr()
        self.attr2.SetBackgroundColour("light blue")

        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemDeselected)
        """
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)

        # keep shift key value
        self.shiftDown = False
        self.totalSelected = 0


    def CreateImageList(self, imgPreview):
        # get thumbnail size
        if imgPreview:
            xy = int(main.bottomWindow.thumbSize.GetStringSelection())
            thumbSize=(xy,xy)
        else:
            thumbSize=(16,16)
        # apply to list
        self.imgs = wx.ImageList(thumbSize[0],thumbSize[1])
        self.folderIco = self.imgs.Add(wx.Bitmap(utils.realPath(u'icons/folder%s.png'%thumbSize[0]),
           wx.BITMAP_TYPE_PNG))
        self.fileIco = self.imgs.Add(wx.Bitmap(utils.realPath(u'icons/file%s.png'%thumbSize[0]),
           wx.BITMAP_TYPE_PNG))
        self.SetImageList(self.imgs, wx.IMAGE_LIST_SMALL)
        return thumbSize

    # gets item info and parses it for inclusion in renaming list
    def GetItemInfo(self, item):
        item_txt = self.GetItemText(item)
        item_txt = item_txt.lstrip(os.sep)
        fullItem = os.path.join(self.pickerPanel.path.GetValue(),item_txt)
        IsFile = os.path.isfile(fullItem)
        fullItem = (fullItem,IsFile)
        return fullItem

    # Need to know if shift is pressed to adjust selection
    def OnKeyDown(self, event):
        if event.m_keyCode == wx.WXK_SHIFT:
            self.shiftDown = True
        else:
            self.shiftDown = False

    def OnKeyUp(self, event):
        self.shiftDown = False
        if event.m_keyCode == wx.WXK_SHIFT:
            self.totalSelected = 0
            self.pickerPanel.enableButtons()
            main.showPreview(True)


    # show selected items and add/remove from renaming list
    def OnItemSelected(self, event):
        currentItem = event.m_itemIndex
        item = wx.ListItem()
        item.SetId(currentItem)
        fullItem = self.GetItemInfo(currentItem)

        # avoid selecting same item twice when using shift key
        if self.shiftDown:
            self.totalSelected += 1

        if self.totalSelected != 1:
            # add to list / show selected:
            if fullItem not in self.pickerPanel.joinedItems:
                item.SetBackgroundColour(utils.HighlightClr)
                item.SetTextColour(utils.HighlightTxtClr)
                self.pickerPanel.joinedItems.append(fullItem)
            # remove from list / show deselected:
            else:
                item.SetBackgroundColour(utils.BackgroundClr)
                item.SetTextColour(utils.TxtClr)
                self.pickerPanel.joinedItems.remove(fullItem)
            # apply highlighting
            self.SetItem(item)

        # Deselect item to avoid user confusion
        main.currentItem = None
        self.Select(currentItem, False)

        if not self.shiftDown:
            self.pickerPanel.enableButtons()
            main.showPreview(True)


    # create thumbnail from image file
    def imageToPil(self,source,thumbSize):
        source = Image.open(source, 'r')
        source.thumbnail(thumbSize, Image.ANTIALIAS)
        image = apply(wx.EmptyImage, source.size)
        image.SetData(source.convert("RGB").tostring())
        return wx.BitmapFromImage(image)

    # add items found to list
    def AddItems(self, root, folders, files):
        self.thumbnails = {}
        supported = ('.png','.jpg','.bmp','.tif','.jpeg',
                     '.tiff','.gif','.ico','.thm')
        imgPreview = main.bottomWindow.imgPreview.GetValue()
        thumbSize = self.thumbSize = self.CreateImageList(imgPreview)

        # list folders first
        folders.sort(key=lambda x: x.lower())
        for i in range(len(folders)):
            self.InsertImageStringItem(i, folders[i], 0)

        # ... then files
        def make_key(f): return (os.path.dirname(f), f.lower())
        files.sort(key=make_key)
        
        i = len(folders)
        for item in files:
            # preview images
            if imgPreview and os.path.splitext(item)[1].lower() in supported:
                fullitem = os.path.join(root,item.lstrip('/'))
                try:
                    thumb = self.imageToPil(fullitem,thumbSize)
                    
                except IOError:
                    img = 1
                else:
                    img = self.imgs.Add(thumb)
                    # link name with number in imageList for main window
                    self.thumbnails[fullitem] = img
            else:
                img = 1
            self.InsertImageStringItem(i, item, img)
            i += 1

    """
    def OnItemActivated(self, event):
        self.currentItem = event.m_itemIndex
        self.log.WriteText("OnItemActivated: %s\nTopItem: %s\n" %
                           (self.GetItemText(self.currentItem), self.GetTopItem()))


    def OnItemDeselected(self, evt):
        print self.items[evt.m_itemIndex]
        #print "OnItemDeselected: %s" % evt.m_itemIndex

    #---------------------------------------------------
    # These methods are callbacks for implementing the
    # "virtualness" of the list...  Normally you would
    # determine the text, attributes and/or image based
    # on values from some external data source, but for
    # this demo we'll just calculate them
    def OnGetItemText(self, item, col):
        return "Item %d, column %s" % (item, col)

    def OnGetItemImage(self, item):
        return self.folderIco


    def OnGetItemAttr(self, item):
        if item % 3 == 1:
            return self.attr1
        elif item % 3 == 2:
            return self.attr2
        else:
            return None
    """

class pickerPanel(wx.Panel):
    def sizer(self):
        PathSizer = self.PathSizer = wx.BoxSizer(wx.HORIZONTAL)
        SelectSizer = self.SelectSizer = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer = self.mainSizer = wx.BoxSizer(wx.VERTICAL)

        PathSizer.Add(self.BROWSE,0,wx.ALL, 5, wx.ALIGN_CENTER)
        PathSizer.Add(self.OK,0,wx.ALL, 5, wx.ALIGN_CENTER)
        PathSizer.Add(self.path,1,wx.ALIGN_CENTER|wx.RIGHT, 5)

        SelectSizer.Add(self.staticText1,0,wx.ALIGN_CENTER)
        SelectSizer.Add(self.not_type,0,wx.ALIGN_CENTER|wx.LEFT,4)
        SelectSizer.Add(self.FilterSel,20,wx.ALIGN_CENTER)
        SelectSizer.Add(self.filterByRE,0,wx.ALIGN_CENTER|wx.LEFT,5)
        SelectSizer.Add(self.ignoreCase,0,wx.ALIGN_CENTER|wx.LEFT,2)
        SelectSizer.Add(self.useLocale,0,wx.ALIGN_CENTER|wx.LEFT,2)
        SelectSizer.Add((-1,-1),1,wx.EXPAND)
        SelectSizer.Add(self.divider1,0,wx.ALIGN_CENTER)
        SelectSizer.Add((-1,-1),1,wx.EXPAND)
        SelectSizer.Add(self.select,0,wx.RIGHT|wx.ALIGN_CENTER,5)
        SelectSizer.Add(self.folders_on,0,wx.ALIGN_CENTER|wx.RIGHT,7)
        SelectSizer.Add(self.files_on,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        SelectSizer.Add(self.walkIt,0,wx.ALIGN_CENTER)
        SelectSizer.Add((-1,-1),1,wx.EXPAND)
        SelectSizer.Add(self.select_all,0,wx.RIGHT,2)
        SelectSizer.Add(self.select_none,0)
        SelectSizer.Add((-1,-1),1,wx.EXPAND)

        mainSizer.Add(self.path_help,0,wx.LEFT|wx.TOP|wx.EXPAND, 6)
        mainSizer.Add(PathSizer,0,wx.EXPAND|wx.ALIGN_CENTER)
        mainSizer.Add(SelectSizer,0,wx.LEFT|wx.EXPAND,6)
        mainSizer.Add(self.selectionArea,1,wx.EXPAND|wx.TOP, 3)

        self.SetSizerAndFit(mainSizer)

    def _init_ctrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PICKERPANEL, name=u'pickerPanel',
              parent=prnt, style=wx.NO_BORDER)

        self.path = wx.TextCtrl(id=wxID_PICKERPANELPATH, name=u'path',
              parent=self, style=wx.TE_PROCESS_ENTER, value='')
        self.path.Bind(wx.EVT_TEXT_ENTER, self.OnOkButton,
              id=wxID_PICKERPANELPATH)

        self.OK = wx.Button(id=wxID_PICKERPANELOK, label=_(u"OK"), name=u'OK',
              parent=self, style=wx.BU_EXACTFIT)
        self.OK.Enable(True)
        self.OK.SetToolTipString(_(u"Load or reload current path"))
        self.OK.Bind(wx.EVT_BUTTON, self.OnOkButton, id=wxID_PICKERPANELOK)

        self.BROWSE = wx.Button(id=wxID_PICKERPANELBROWSE, label=_(u"Browse"),
          name=u'BROWSE', parent=self, style=wx.BU_EXACTFIT)
        self.BROWSE.SetToolTipString(_(u"Browse for path"))
        self.BROWSE.Bind(wx.EVT_BUTTON, self.OnBROWSEButton,
          id=wxID_PICKERPANELBROWSE)

        self.path_help = wx.StaticText(id=wxID_PICKERPANELPATH_HELP,
              label=_(u"Input/Paste path and press OK, or BROWSE for path:"),
              name=u'path_help', parent=self, style=0)

        self.select = wx.StaticText(id=wxID_PICKERPANELSELECT, label=_(u"Select:"),
              name=u'select', parent=self, style=0)

        self.select_all = wx.Button(id=wxID_PICKERPANELSELECT_ALL, label=_(u"all"),
              name=u'select_all', parent=self, style=wx.BU_EXACTFIT)
        self.select_all.Enable(False)
        self.select_all.Bind(wx.EVT_BUTTON, self.OnSelect_all,
              id=wxID_PICKERPANELSELECT_ALL)

        self.select_none = wx.Button(id=wxID_PICKERPANELSELECT_NONE,
              label=_(u"none"), name=u'select_none', parent=self, style=wx.BU_EXACTFIT)
        self.select_none.Enable(False)
        self.select_none.Bind(wx.EVT_BUTTON, self.OnSelect_none,
              id=wxID_PICKERPANELSELECT_NONE)

        self.folders_on = wx.CheckBox(id=wxID_PICKERPANELFOLDERSON, label=_(u"Folders"),
              name=u'folders_on', parent=self, style=0,)
        self.folders_on.SetValue(True)
        self.folders_on.Bind(wx.EVT_CHECKBOX, self.ShowPicker,
              id=wxID_PICKERPANELFOLDERSON)

        self.files_on = wx.CheckBox(id=wxID_PICKERPANELFILESON, label=_(u"Files"),
              name=u'files_on', parent=self, style=0)
        self.files_on.SetValue(True)
        self.files_on.Bind(wx.EVT_CHECKBOX, self.ShowPicker,
              id=wxID_PICKERPANELFILESON)

        self.FilterSel = wx.ComboBox(choices=self.CommonSearches.keys(), id=wxID_PICKERPANELFILTERSEL,
              name=u'FilterSel', parent=self, style=wx.TE_PROCESS_ENTER)
        self.FilterSel.SetSelection(0)
        self.FilterSel.SetToolTipString(_(u"Names containing (Use menu or enter text)"))
        self.FilterSel.Bind(wx.EVT_COMBOBOX, self.OnFilterSel,
              id=wxID_PICKERPANELFILTERSEL)
        self.FilterSel.Bind(wx.EVT_TEXT_ENTER, self.ShowPicker,
              id=wxID_PICKERPANELFILTERSEL)


        self.staticText1 = wx.StaticText(id=wxID_PICKERPANELSTATICTEXT1,
              label=_(u"Filter:"), name=u'staticText1', parent=self, style=0)

        self.filterByRE = wx.CheckBox(id=wxID_PICKERPANELFILTERBYRE, label=_(u"Reg-Expr"),
              name=u'filterByRE', parent=self, style=0)
        self.filterByRE.SetValue(False)
        self.filterByRE.SetToolTipString(_(u"Evaluate filter as a regular expression"))
        self.filterByRE.Bind(wx.EVT_CHECKBOX, self.REoptions,
              id=wxID_PICKERPANELFILTERBYRE)

        self.ignoreCase = wx.CheckBox(id=wxID_PICKERPANELIGNORECASE, label=_(u"I"),
              name=u'ignoreCase', parent=self, style=0)
        self.ignoreCase.SetValue(True)
        self.ignoreCase.Enable(False)
        self.ignoreCase.SetToolTipString(_(u"case-Insensitive match"))
        self.ignoreCase.Bind(wx.EVT_CHECKBOX, self.ShowPicker,
              id=wxID_PICKERPANELIGNORECASE)

        self.useLocale = wx.CheckBox(id=wxID_PICKERPANELUSELOCALE, label=_(u"U"),
              name=u'useLocale', parent=self, style=0)
        self.useLocale.SetValue(True)
        self.useLocale.Enable(False)
        self.useLocale.SetToolTipString(_(u"Unicode match (\w matches 'a','b','c', etc)"))
        self.useLocale.Bind(wx.EVT_CHECKBOX, self.ShowPicker,
              id=wxID_PICKERPANELUSELOCALE)

        self.walkIt = wx.CheckBox(id=wxID_PICKERPANELWALKIT, label=_(u"Walk"),
              name=u'walkIt', parent=self, style=0)
        self.walkIt.SetValue(False)
        self.walkIt.SetToolTipString(_(u"Get all files in directory and sub-directories, but no folders"))
        self.walkIt.Bind(wx.EVT_CHECKBOX, self.OnWalkItCheckbox,
              id=wxID_PICKERPANELWALKIT)

        self.not_type = wx.CheckBox(id=wxID_PICKERPANELNOT_TYPE, label=u'!= ',
              name=u'not_type', parent=self, style=0)
        self.not_type.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.BOLD, False))
        self.not_type.SetValue(False)
        self.not_type.SetToolTipString(_(u"NOT containing"))
        self.not_type.Bind(wx.EVT_CHECKBOX, self.ShowPicker,
              id=wxID_PICKERPANELNOT_TYPE)

        self.divider1 = wx.StaticLine(id=wxID_PICKERPANELDIVIDER1,
              name=u'divider1', parent=self, size=wx.Size(3, 22), style=wx.LI_VERTICAL)

        # create split window:
        self.selectionArea = wx.SplitterWindow(parent=self, id=wxID_SELECTIONAREA,
          name=u'selectionArea', style=wx.SP_LIVE_UPDATE)

        # directory picker:
        if main.prefs[u'useDirTree']:
            self.createDirTree()

        # file picker:
        self.picker = picker(id=wxID_SELECTIONAREAFPICKER, name=u'picker',
              parent=self.selectionArea)

        self.initSplitter()


    def __init__(self, parent, main_window):
        global parnt
        parnt = parent
        global main
        main = main_window
        global root
        root = ''

        self.CommonSearches = {"" : "",
             _(u"- audio") : u"\.(mp3|wav|ogg|flac|wma|aiff|aac)$",
             _(u"- image") : u"\.(bmp|jpg|jpeg|png|svg|ico|tif|tiff|gif|psd|thm|nef)$",
             _(u"- video") : u"\.(avi|mpg|mpeg|mpe|mp4|m4e|wmv|divx|flc|mov|ogm)$",
             _(u"- office related") : u"\.(txt|csv|rtf|doc|otf|xls|pps)$",
             _(u"- web related") : u"\.(htm|html|php|css|js|asp|cgi|swf)$",
             _(u"- programming") : u"\.(py|pyc|php|pl|h|c|cpp|cxx|jar|java|js|tcl)$",
             _(u"- compressed") : u"\.(zip|tar|7z|ace|gz|tgz|rar|r\d{1,3}|cab|bz2)$",
            #_(u"- ") : u"\.()$",
            #_(u"- ") : u"\.()$",
                              }
        self.CustomSearches = {}

        self.joinedItems = []
        self._init_ctrls(parent)
        
        # adjust button sizes
        utils.adjustExactbuttons(self)
        self.sizer()


    # Adjust after loading config
    def onConfigLoad(self,):
        self.REoptions(False)
        self.OnWalkItCheckbox(False)

    # ---- create the directory tree:
    def createDirTree(self):
        self.dirPicker = DirControl(self.selectionArea,
                                    wxID_SELECTIONAREADIRPICKER,
                                    u'dirPicker',
                                    main.prefs['showHiddenDirs'])
        self.dirPicker.bind()
            

    def initSplitter(self):
        # initialise split window based on preferences:
        if main.prefs['useDirTree']:
            self.selectionArea.SplitVertically(self.dirPicker, self.picker, 280)
        else:
            self.selectionArea.Initialize(self.picker)

    # ---- hide/show directory tree:
    def setTree(self):
        if main.prefs[u'useDirTree']:
            self.createDirTree()
            self.dirPicker.SetPath(self.path.GetValue())
        else:
            self.selectionArea.Unsplit(self.dirPicker)
            self.dirPicker.Destroy()
        self.initSplitter()


    #---- get and set path:
    def OnBROWSEButton(self, event):
        dlg = wx.DirDialog(self,_(u"Choose a directory:"),
            style=wx.DD_DEFAULT_STYLE)
        dlg.SetPath(self.path.GetValue())
        try:
            if dlg.ShowModal() == wx.ID_OK:
                dir = dlg.GetPath()
                self.path.SetValue(dir)
                self.OnOkButton(event)
        finally: dlg.Destroy()


    #---- set path:
    def OnOkButton(self, event):
        if main.prefs[u'useDirTree']:
            # keep a record of the original path
            path = self.path.GetValue()
            # need to temporarily unbind to avoid showing picker twice
            self.dirPicker.Bind(wx.EVT_TREE_SEL_CHANGED, utils.devNull)
            try:
                self.dirPicker.SetPath(path)
            except AttributeError:
                pass
            # OK bind again
            self.dirPicker.Bind(wx.EVT_TREE_SEL_CHANGED, self.dirPicker.dirChange)
            # if tree won't display original path, force it
            if self.dirPicker.GetPath() != path:
                self.path.SetValue(path)
                self.ShowPicker(event)
            else:
                self.ShowPicker(event)
        # always refresh if dirpicker not there
        else:
            self.ShowPicker(event)


    #---- set common searches
    def OnFilterSel(self, event):
        filter = self.FilterSel
        if self.CommonSearches.has_key(filter.GetStringSelection()):
            checkboxes = (self.filterByRE, self.ignoreCase, self.useLocale)
            for chckbx in checkboxes:
                chckbx.SetValue(True)
                chckbx.Enable(True)
        else:
            filter.SetLabel(u'')
            filter.SetValue(u'')
        self.ShowPicker(event)


    # add all items to renaming list
    def OnSelect_all(self, event):
        # clear list
        self.joinedItems = []
        for i in range(self.picker.GetItemCount()):
            # add to list
            self.joinedItems.append(self.picker.GetItemInfo(i))
            # show selected in picker
            item = wx.ListItem()
            item.SetId(i)
            #item.SetState(wx.LIST_STATE_SELECTED|wx.LIST_STATE_FOCUSED)
            item.SetBackgroundColour(utils.HighlightClr)
            item.SetTextColour(utils.HighlightTxtClr)
            self.picker.SetItem(item)
        # enable buttons
        self.picker.Refresh()
        self.select_none.Enable(True)
        main.menuPicker.getNoneMenu.Enable(True)
        self.select_all.Enable(False)
        main.menuPicker.getAllMenu.Enable(False)
        main.currentItem = None
        main.showPreview(event)

    # remove all items from renaming list
    def OnSelect_none(self, event):
        total_items = self.picker.GetItemCount()
        # clear list
        self.joinedItems = []
        # show deselected in picker
        for i in range(total_items):
            item = wx.ListItem()
            item.SetId(i)
            item.SetBackgroundColour(utils.BackgroundClr)
            item.SetTextColour(utils.TxtClr)
            self.picker.SetItem(item)
        # enable buttons
        self.select_all.Enable(True)
        main.menuPicker.getAllMenu.Enable(True)
        self.select_none.Enable(False)
        main.menuPicker.getNoneMenu.Enable(False)
        main.currentItem = None
        main.notebook.GetPage(3).clear_errors()
        main.showPreview(event)

    # enable buttons (also called by error.py)
    def enableButtons(self):
        total_items = self.picker.GetItemCount()
        if self.joinedItems != []:
            self.select_none.Enable(True)
            main.menuPicker.getNoneMenu.Enable(False)
            if len(self.joinedItems) == total_items:
                self.select_all.Enable(False)
                main.menuPicker.getAllMenu.Enable(False)
            else:
                self.select_all.Enable(True)
                main.menuPicker.getAllMenu.Enable(True)
        else:
            self.select_all.Enable(True)
            main.menuPicker.getAllMenu.Enable(True)
            self.select_none.Enable(False)
            main.menuPicker.getNoneMenu.Enable(False)


    def ShowPicker(self, event):
        """
        This is the most important function for the picker.
        It will go and grab items from the specified directory,
        either as a listing or as a walk, and filter out entries
        based on user settings. Files and folders are seperated
        for proper sorting.
        This function is called, depending on user preferences,
        from almost every widget in the picker panel, and from
        main_window.py.
        """
        self.clearALL()
        files = [] #files will go here
        folders = [] #folders will go here
        err = False
        i = 0
        root = unicode(self.path.GetValue())

        # don't do anything for blank path
        if not root:
            main.setStatusMsg('',u'eyes')
            main.bottomWindow.display.DeleteAllItems()

        # don't do anything unless path is readable
        elif not os.access(root, os.R_OK):
            main.setStatusMsg(_(u"Cannot read path!"),u'warn')
            main.bottomWindow.display.DeleteAllItems()

        # OK, load items up:
        else:
            wx.BeginBusyCursor()
            self.select_none.Enable(False)
            main.menuPicker.getNoneMenu.Enable(False)
            main.bottomWindow.go.Enable(False)
            main.setStatusMsg(_(u"Getting directory contents please wait ..."),u'wait')
            not_type = self.not_type.GetValue()
            folderOn = self.folders_on.GetValue()
            fileOn = self.files_on.GetValue()

            # are we using a built in filter?
            filterSel = self.FilterSel.GetValue()
            if self.CommonSearches.has_key(filterSel):
                filter = self.CommonSearches[filterSel]
            else:
                filter = filterSel
            filterRE = self.filterByRE.GetValue()

            # are we searching by regular expression?
            if filter and filterRE:
                ignoreCase = self.ignoreCase.GetValue()
                useLocale = self.useLocale.GetValue()
                try:
                    # compile according to options:
                    if ignoreCase and useLocale:
                        filter = re.compile(filter, re.IGNORECASE | re.UNICODE)
                    elif ignoreCase:
                        filter = re.compile(filter, re.IGNORECASE)
                    elif useLocale:
                        filter = re.compile(filter, re.UNICODE)
                    else:
                        filter = re.compile(filter)
                except sre_constants.error, err:
                    main.setStatusMsg(_(u"Regular-Expression: %s")%err,u'warn')
                    main.REmsg = True
                    filter = re.compile(r'')
                    pass
                useRE = True
            else:
                filter = filter.lower()
                useRE = False

            # create the search (filtering) operations...
            # normal filtering:
            if filter and not useRE:
                def filterFolders(entry):
                    if filter in entry.lower() and not not_type:
                        folders.append(entry)
                    if filter not in entry.lower() and not_type:
                        folders.append(entry)
                def filterFiles(entry):
                    if filter in entry.lower() and not not_type:
                        files.append(entry)
                    if filter not in entry.lower() and not_type:
                        files.append(entry)
            # regular expression filtering
            elif filter and useRE:
                def filterFolders(entry):
                    if filter.search(entry) and not not_type:
                        folders.append(entry)
                    if not filter.search(entry) and not_type:
                        folders.append(entry)
                def filterFiles(entry):
                    if filter.search(entry) and not not_type:
                        files.append(entry)
                    if not filter.search(entry) and not_type:
                        files.append(entry)
            # no filtering:
            else:
                def filterFolders(entry):
                    folders.append(entry)
                def filterFiles(entry):
                    files.append(entry)

            # Now to get the items according to operations defined above...
            # retrieve items by walking:
            if self.walkIt.GetValue():
                try:
                    for base, dirs, walk_files in os.walk(root):
                        #main.Update()
                        base = base.replace(root,'')
                        #XXX TODO allow to retrieve folders
                        #for entry in dirs:
                        #    entry = os.path.join(base,entry)
                        #    filterFolders(entry)
                        for entry in walk_files:
                            entry = os.path.join(base,entry)
                            filterFiles(entry)

                except UnicodeDecodeError, err:
                    utils.makeErrMsg(unicode(err), _("Unable to load item"))
                    err = True
                    pass
                except WindowsError:
                    main.setStatusMsg(_(u"Cannot read path!"),u'warn')
                    err = True
                    pass
                else:
                    main.setStatusMsg(_(u"Retrieved %s items from directory")%len(files),u'wait')

            # normal retrieval:
            else:
                # Loop through items in directory:
                try:
                    listedDir = os.listdir(root)
                except WindowsError:
                    main.setStatusMsg(_(u"Cannot read path!"),u'warn')
                    err = True
                    pass
                else:
                    for entry in listedDir:
                        try:
                            isFolder = os.path.isdir(os.path.join(root, entry))
                        except UnicodeDecodeError, err:
                            entry = entry.decode(sys.getfilesystemencoding(), 'replace')
                            msg = _("%s\n%s\n\nAttempt to load more items from this directory?")%(entry,err)
                            dlg = wx.MessageDialog(self, msg, _("Unable to load item"), wx.YES_NO|wx.ICON_ERROR)
                            if dlg.ShowModal() == wx.ID_YES:
                                pass
                            else:
                                break
                        #load folders if set:
                        if folderOn and isFolder:
                            filterFolders(entry)
                        #load files if set:
                        if fileOn and not isFolder:
                            filterFiles(entry)

            if not err:
                self.picker.AddItems(root, folders, files)
                main.setStatusMsg(_(u"Retrieved %s items from directory")%self.picker.GetItemCount(),
                   u'complete')

                # after retrieval:
                self.select_all.Enable(True)
                main.menuPicker.getAllMenu.Enable(True)
            
            main.bottomWindow.display.DeleteAllItems()
            wx.EndBusyCursor()
            main.Refresh()

            if main.prefs[u'autoSelectAll']:
                self.OnSelect_all(event)


    #---- Sorted retrieval of items.
    def GetItems(self):
        sortPanel = main.notebook.GetPage(2)

        # sorting by section
        if sortPanel.byPosition.GetValue():
            s = sortPanel.PosStart.GetValue()
            l = sortPanel.PosLength.GetValue()
            frm,to = utils.CalcStartEndPos(s,l)
            def foldersKey(x):
                return os.path.basename(x[0])[frm:to].lower()
            def filesKey(x):
                dir, file = os.path.split(x[0])
                return (dir, file[frm:to].lower())

        # inteligent sorting
        elif sortPanel.intelySort.GetValue():
            # return the number in the name
            def getNumb(x):
                try:
                    compare = float(re.sub('\D','',x[0]))
                except ValueError:
                    compare = x[0]
                return compare
            # sort by the number
            def foldersKey(x):
                return getNumb(x)
            def filesKey(x):
                compare = getNumb(x)
                return (os.path.dirname(x[0]), compare)

        # sort by item attributes
        elif sortPanel.statSort.GetValue():
            # choices are in different order than os.stat
            ref = ('st_ctime', 'st_mtime', 'st_atime', 'st_size', 'st_mode',
                   'st_uid', 'st_gid', 'st_ino', 'st_nlink', 'st_dev')
            s = ref[sortPanel.statSortChoice.GetSelection()]
            def foldersKey(x):
                stat = os.stat(x[0])
                return getattr(stat, s)
            def filesKey(x):
                stat = os.stat(x[0])
                return getattr(stat, s)
        
        # normal sorting
        else:
            def foldersKey(x):
                return x[0].lower()
            def filesKey(x):
                return (os.path.dirname(x[0]), x[0].lower())

        # sort items automatically?
        if not sortPanel.manually.GetValue():
            main.currentItem = None
            self.fc = 0
            def dirsTop(f):
                if f[1] == 0:
                    self.fc += 1
                return f[1]
            def dirsBottom(f):
                if f[1] == 0:
                    return 1
                else:
                    self.fc += 1
                    return 0

            # place directories where in list?
            if sortPanel.dirsPlace.GetSelection() == 0:
                self.joinedItems.sort(key=dirsTop)
            if sortPanel.dirsPlace.GetSelection() == 1:
                self.joinedItems.sort(key=dirsBottom)

            # seperate files from folders
            folders = self.joinedItems[:self.fc]
            files = self.joinedItems[self.fc:]

            # sort both
            folders.sort(key=foldersKey)
            files.sort(key=filesKey)

            # join back together
            self.joinedItems[:self.fc] = folders
            self.joinedItems[self.fc:] = files

            # reverse sort list?
            if sortPanel.descending.GetValue():
                self.joinedItems.reverse()

        return self.joinedItems


    #---- clear stuff out
    def clearALL(self):
        self.joinedItems = []
        self.picker.DeleteAllItems()

    def walkFromMenu(self, event):
        self.walkIt.SetValue(True)
        self.OnWalkItCheckbox(event)

    def OnWalkItCheckbox(self, event):
        if self.walkIt.GetValue():
            self.folders_on.Enable(False)
            self.files_on.Enable(False)
        else:
            self.folders_on.Enable(True)
            self.files_on.Enable(True)
        if event:
            self.ShowPicker(event)

    def REoptions(self, event):
        if self.filterByRE.GetValue():
            self.ignoreCase.Enable(True)
            self.useLocale.Enable(True)
        else:
            self.ignoreCase.Enable(False)
            self.useLocale.Enable(False)
        if event:
            self.ShowPicker(event)
